# Stubs for zope.schema._bootstrapfields (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import threading
from typing import Any, Optional
from zope.interface import Attribute
from zope.schema._bootstrapinterfaces import ValidationError

__docformat__: str

class _NotGiven: ...

class ValidatedProperty:
    def __init__(self, name: Any, check: Optional[Any] = ..., allow_none: bool = ...) -> None: ...
    def __set__(self, inst: Any, value: Any) -> None: ...
    def __get__(self, inst: Any, owner: Any): ...

class DefaultProperty(ValidatedProperty):
    def __get__(self, inst: Any, owner: Any): ...

def getFields(schema: Any): ...

class _DocStringHelpers:
    @staticmethod
    def docstring_to_lines(docstring: Any): ...
    @staticmethod
    def make_class_directive(kind: Any): ...
    @classmethod
    def make_field(cls, name: Any, value: Any): ...
    @classmethod
    def make_class_field(cls, name: Any, kind: Any): ...

class Field(Attribute):
    context: Any = ...
    missing_value: Any = ...
    order: int = ...
    default: Any = ...
    __name__: Any = ...
    interface: Any = ...
    title: Any = ...
    description: Any = ...
    required: Any = ...
    readonly: Any = ...
    constraint: Any = ...
    defaultFactory: Any = ...
    def __init__(self, title: str = ..., description: str = ..., __name__: str = ..., required: bool = ..., readonly: bool = ..., constraint: Optional[Any] = ..., default: Optional[Any] = ..., defaultFactory: Optional[Any] = ..., missing_value: Any = ...) -> None: ...
    def constraint(self, value: Any): ...
    def bind(self, context: Any): ...
    def validate(self, value: Any) -> None: ...
    def __hash__(self): ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    def get(self, object: Any): ...
    def query(self, object: Any, default: Optional[Any] = ...): ...
    def set(self, object: Any, value: Any) -> None: ...
    def getExtraDocLines(self): ...
    def getDoc(self): ...

class Container(Field): ...
class Iterable(Container): ...

class Orderable:
    min: Any = ...
    max: Any = ...
    default: Any = ...
    def __init__(self, min: Optional[Any] = ..., max: Optional[Any] = ..., default: Optional[Any] = ..., **kw: Any) -> None: ...

class MinMaxLen:
    min_length: int = ...
    max_length: Any = ...
    def __init__(self, min_length: int = ..., max_length: Optional[Any] = ..., **kw: Any) -> None: ...

class Text(MinMaxLen, Field):
    def __init__(self, *args: Any, **kw: Any) -> None: ...
    def fromUnicode(self, str: Any): ...

class TextLine(Text):
    def constraint(self, value: Any): ...

class Password(TextLine):
    UNCHANGED_PASSWORD: Any = ...
    def set(self, context: Any, value: Any): ...
    def validate(self, value: Any): ...

class Bool(Field):
    def set(self, object: Any, value: Any) -> None: ...
    def fromUnicode(self, value: Any): ...
    def fromBytes(self, value: Any): ...

class InvalidNumberLiteral(ValueError, ValidationError): ...

class Number(Orderable, Field):
    def fromUnicode(self, value: Any): ...
    fromBytes: Any = ...
    def fromBytes(self, value: Any): ...

class Complex(Number): ...
class Real(Complex): ...
class Rational(Real): ...
class InvalidIntLiteral(ValueError, ValidationError): ...
class Integral(Rational): ...
class Int(Integral): ...

class _ObjectsBeingValidated(threading.local):
    ids_being_validated: Any = ...
    def __init__(self) -> None: ...

def get_schema_validation_errors(schema: Any, value: Any, _validating_objects: Any = ...): ...
def get_validation_errors(schema: Any, value: Any, validate_invariants: bool = ...): ...

class Object(Field):
    schema: Any = ...
    validate_invariants: Any = ...
    def __init__(self, schema: Any = ..., **kw: Any) -> None: ...
    def getExtraDocLines(self): ...
    def set(self, object: Any, value: Any) -> None: ...

class BeforeObjectAssignedEvent:
    object: Any = ...
    name: Any = ...
    context: Any = ...
    def __init__(self, object: Any, name: Any, context: Any) -> None: ...



# Stubs for zope.interface.interface (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

CO_VARARGS: int
CO_VARKEYWORDS: int
TAGGED_DATA: str

def invariant(call: Any): ...
def taggedValue(key: Any, value: Any): ...

class Element:
    __name__: Any = ...
    __doc__: Any = ...
    def __init__(self, __name__: Any, __doc__: str = ...) -> None: ...
    def getName(self): ...
    def getDoc(self): ...
    def getTaggedValue(self, tag: Any): ...
    def queryTaggedValue(self, tag: Any, default: Optional[Any] = ...): ...
    def getTaggedValueTags(self): ...
    def setTaggedValue(self, tag: Any, value: Any) -> None: ...

class SpecificationBasePy:
    def providedBy(self, ob: Any) -> bool: ...
    def implementedBy(self, cls: Any) -> bool: ...
    def isOrExtends(self, interface: Any) -> bool: ...
    __call__: Any = ...
SpecificationBase = SpecificationBasePy

class InterfaceBasePy:
    def __call__(self, obj: Any, alternate: Any = ...): ...
    def __adapt__(self, obj: Any): ...
InterfaceBase = InterfaceBasePy
adapter_hooks: Any

class Specification(SpecificationBase):
    dependents: Any = ...
    __bases__: Any = ...
    def __init__(self, bases: Any = ...) -> None: ...
    def subscribe(self, dependent: Any) -> None: ...
    def unsubscribe(self, dependent: Any) -> None: ...
    __sro__: Any = ...
    __iro__: Any = ...
    def changed(self, originally_changed: Any) -> None: ...
    def interfaces(self) -> None: ...
    def extends(self, interface: Any, strict: bool = ...): ...
    def weakref(self, callback: Optional[Any] = ...): ...
    def get(self, name: Any, default: Optional[Any] = ...): ...

class InterfaceClass(type, Element, InterfaceBase, Specification):
    __module__: Any = ...
    __identifier__: Any = ...
    def __init__(self, name: Any, bases: Any = ..., attrs: Optional[Any] = ..., __doc__: Optional[Any] = ..., __module__: Optional[Any] = ...) -> None: ...
    def interfaces(self) -> None: ...
    def getBases(self): ...
    def isEqualOrExtendedBy(self, other: Any): ...
    def names(self, all: bool = ...): ...
    def __iter__(self): ...
    def namesAndDescriptions(self, all: bool = ...): ...
    def getDescriptionFor(self, name: Any): ...
    def __getitem__(self, key, default: Optional[Any] = ...) -> Any: ...
    def __contains__(self, name: Any): ...
    def direct(self, name: Any): ...
    def queryDescriptionFor(self, name: Any, default: Optional[Any] = ...): ...
    def validateInvariants(self, obj: Any, errors: Optional[Any] = ...) -> None: ...
    def __reduce__(self): ...
    def __hash__(self): ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    def __lt__(self, other: Any): ...
    def __le__(self, other: Any): ...
    def __gt__(self, other: Any): ...
    def __ge__(self, other: Any): ...

class Interface(metaclass=InterfaceClass): ...

class Attribute(Element):
    interface: Any = ...

class Method(Attribute):
    positional: Any = ...
    required: Any = ...
    varargs: Any = ...
    kwargs: Any = ...
    optional: Any = ...
    def __call__(self, *args: Any, **kw: Any) -> None: ...
    def getSignatureInfo(self): ...
    def getSignatureString(self): ...

def fromFunction(func: Any, interface: Optional[Any] = ..., imlevel: int = ..., name: Optional[Any] = ...): ...
def fromMethod(meth: Any, interface: Optional[Any] = ..., name: Optional[Any] = ...): ...



"""Interface can define a __getattr__ method
"""
import zope.interface

class IData(zope.interface.Interface):
    def __getattr__(name: str) -> int:
        pass

def main() -> None:
    data = IData(None)
    reveal_type(data.legs)

"""
<output>
interface_getattr.py:11: note: Revealed type is "builtins.int"
</output>
"""



"""When interface is imported from unknown package (one without stubs),
do not produce errors.
"""

import zope.interface

from unknown import interfaces

@zope.interface.implementer(interfaces.IUnknownInterface)
class Bookmark(object):
    pass

def main() -> None:
    bm: interfaces.IUnknownInterface = Bookmark()
    reveal_type(bm)


"""
<output>
interface_unknown.py:7: error: Cannot find implementation or library stub for module named "unknown"
interface_unknown.py:7: note: See https://mypy.readthedocs.io/en/stable/running_mypy.html#missing-imports
interface_unknown.py:15: note: Revealed type is "Any"
</output>
"""



"""Interface contains a method without arguments.

Mypy should not complain.
"""
import zope.interface


class ISomething(zope.interface.Interface):
    def hello() -> None:
        pass

    def add(a, b) -> int:
        pass


@zope.interface.implementer(ISomething)
class Something(object):
    def hello(self) -> None:
        print("Hello world!")

    def add(self, a, b):
        return a + b


def run(smth: ISomething):
    smth.hello()
    smth.add(2, 4)


def main() -> None:
    smth = Something()
    run(smth)


if __name__ == '__main__':
    main()

"""
<output>
</output>
"""



import zope.interface
from typing import Type, TypeVar

T = TypeVar('T', bound='zope.interface.Interface')


class IBookmark(zope.interface.Interface):
    pass

def createOb(iface: Type[T]) -> T:
    if zope.interface.interfaces.IInterface.providedBy(iface):
        pass
    return iface(None)

def main(self) -> None:
    bm = createOb(IBookmark)
    reveal_type(bm)

"""
<output>
interface_meta.py:17: note: Revealed type is "__main__.IBookmark"
</output>
"""



# Stubs for zope.schema._field (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from zope.schema._bootstrapfields import Field as Field, Bool as Bool, TextLine as TextLine, Complex as Complex, Int as Int, Real as Real, Object as Object
from zope.schema._bootstrapfields import Iterable as Iterable, MinMaxLen as MinMaxLen, Number as Number, Orderable as Orderable, Text as Text
from zope.schema.interfaces import ValidationError

__docformat__: str

class SourceText(Text):
    __doc__: Any = ...

class Bytes(MinMaxLen, Field):
    __doc__: Any = ...
    def fromUnicode(self, value: Any): ...
    def fromBytes(self, value: Any): ...

class NativeString(Text):
    def fromBytes(self, value: Any): ...

class ASCII(NativeString):
    __doc__: Any = ...

class BytesLine(Bytes):
    def constraint(self, value: Any): ...

class NativeStringLine(TextLine):
    def fromBytes(self, value: Any): ...

class ASCIILine(ASCII):
    __doc__: Any = ...
    def constraint(self, value: Any): ...

class InvalidFloatLiteral(ValueError, ValidationError): ...
class Float(Real): ...
class InvalidDecimalLiteral(ValueError, ValidationError): ...
class Decimal(Number): ...

class Datetime(Orderable, Field):
    __doc__: Any = ...
    def __init__(self, *args: Any, **kw: Any) -> None: ...

class Date(Orderable, Field):
    __doc__: Any = ...

class Timedelta(Orderable, Field):
    __doc__: Any = ...

class Time(Orderable, Field):
    __doc__: Any = ...

class MissingVocabularyError(ValidationError, ValueError, LookupError): ...

class InvalidVocabularyError(ValidationError, ValueError, TypeError):
    def __init__(self, vocabulary: Any) -> None: ...

class Choice(Field):
    vocabulary: Any = ...
    vocabularyName: Any = ...
    def __init__(self, values: Optional[Any] = ..., vocabulary: Optional[Any] = ..., source: Optional[Any] = ..., **kw: Any) -> None: ...
    source: Any = ...
    def bind(self, context: Any): ...
    def fromUnicode(self, value: Any): ...

class _StrippedNativeStringLine(NativeStringLine):
    def fromUnicode(self, value: Any): ...
    def fromBytes(self, value: Any): ...

class URI(_StrippedNativeStringLine): ...
class PythonIdentifier(_StrippedNativeStringLine): ...

class DottedName(_StrippedNativeStringLine):
    min_dots: Any = ...
    max_dots: Any = ...
    def __init__(self, *args: Any, **kw: Any) -> None: ...

class Id(_StrippedNativeStringLine): ...

class InterfaceField(Field):
    __doc__: Any = ...

class Collection(MinMaxLen, Iterable):
    value_type: Any = ...
    unique: bool = ...
    def __init__(self, value_type: Any = ..., unique: Any = ..., **kw: Any) -> None: ...
    def bind(self, context: Any): ...
AbstractCollection = Collection

class Sequence(Collection): ...
class Tuple(Sequence): ...
class MutableSequence(Sequence): ...
class List(MutableSequence): ...

class _AbstractSet(Collection):
    unique: bool = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class Set(_AbstractSet): ...
class FrozenSet(_AbstractSet): ...

class Mapping(MinMaxLen, Iterable):
    key_type: Any = ...
    value_type: Any = ...
    def __init__(self, key_type: Optional[Any] = ..., value_type: Optional[Any] = ..., **kw: Any) -> None: ...
    def bind(self, object: Any): ...

class MutableMapping(Mapping): ...
class Dict(MutableMapping): ...



import os
import re

import pytest
from mypy import build
from mypy import options
from mypy.errors import CompileError
from mypy.modulefinder import BuildSource


HERE = os.path.abspath(os.path.dirname(__file__))
SAMPLES_DIR = os.path.join(HERE, "samples")
OUTPUT_RE = re.compile(r"<output>(.*)</output>", re.MULTILINE | re.DOTALL)


@pytest.fixture(scope="session")
def mypy_cache_dir(tmp_path_factory):
    tdir = tmp_path_factory.mktemp('.mypy_cahe')
    print("Setup cache", str(tdir))
    return str(tdir)


def test_samples(samplefile, mypy_cache_dir):
    opts = options.Options()
    opts.cache_dir = mypy_cache_dir
    opts.show_traceback = True
    opts.namespace_packages = True
    opts.hide_error_codes = True
    opts.plugins = ['mypy_zope:plugin']
    opts.python_version = (3, 11)
    # Config file is needed to load plugins, it doesn't not exist and is not
    # supposed to.
    opts.config_file = '    not_existing_config.ini'

    try:
        base_dir = os.path.dirname(samplefile)
        source = BuildSource(samplefile,
                             module=None,
                             text=None,
                             base_dir=base_dir)
        res = build.build(
            sources=[source],
            options=opts)
    except CompileError as e:
        assert False, e

    normalized = normalize_errors(res.errors, samplefile)
    actual = '\n'.join(normalized)
    expected = find_expected_output(samplefile)
    assert actual == expected


def find_expected_output(filename):
    with open(filename, "r") as f:
        source = f.read()
        m = OUTPUT_RE.search(source)
        if not m:
            return ""
        return m.group(1).strip()


def normalize_errors(errors, filename):
    cwd = os.getcwd() + os.path.sep
    assert filename.startswith(cwd)
    relfname = filename[len(cwd):]
    basename = os.path.basename(filename)
    return [msg.replace(relfname, basename) for msg in errors]


def pytest_generate_tests(metafunc):
    samples = []
    ids = []
    for fname in os.listdir(SAMPLES_DIR):
        samples.append(os.path.join(SAMPLES_DIR, fname))
        name, ext = os.path.splitext(fname)
        ids.append(name)

    metafunc.parametrize("samplefile", samples, ids=ids)



import zope.interface
from zope.interface.common import mapping

class IBookmarkContainer(mapping.IMapping):
    pass

@zope.interface.implementer(IBookmarkContainer)
class BadContainer(object):
    def values(self):
        pass

@zope.interface.implementer(IBookmarkContainer)
class GoodContainer(dict):
    pass


def main() -> None:
    bc: IBookmarkContainer = GoodContainer()
    bc['one'] = 1

if __name__ == '__main__':
    main()

"""
<output>
interface_mapping.py:8: error: 'BadContainer' is missing following 'zope.interface.common.mapping.IItemMapping' interface members: __getitem__.
interface_mapping.py:8: error: 'BadContainer' is missing following 'zope.interface.common.mapping.IReadMapping' interface members: __contains__, get.
interface_mapping.py:8: error: 'BadContainer' is missing following 'zope.interface.common.mapping.IEnumerableMapping' interface members: __iter__, __len__, items, keys.
interface_mapping.py:8: error: 'BadContainer' is missing following 'zope.interface.common.mapping.IWriteMapping' interface members: __delitem__, __setitem__.
</output>
"""



import zope.interface
import zope.schema

class Bookmark(object):
    # This field is not used inside an interface. It will be treated as
    # zope.schema.TextLine type
    textline = zope.schema.TextLine(
        title='Title',
        description='Optional text line',
        required=False)


def main() -> None:
    bm = Bookmark()
    bm.textline = "text"  # Error, zope.schema.TextLine is expected

if __name__ == '__main__':
    main()

"""
<output>
schema_field_outside_inteface.py:15: error: Incompatible types in assignment (expression has type "str", variable has type "TextLine")
</output>
"""



"""A simple valid interface declaration
"""
import zope.interface


class IAwesome(zope.interface.Interface):
    def sparkle(count: int) -> None: pass

class IClothing(zope.interface.Interface):
    def wear() -> None: pass

@zope.interface.implementer(IAwesome)
class Awesome:
    def sparkle(self, count: int) -> None: pass

@zope.interface.implementer(IClothing)
class Boots:
    def __init__(self, color: str) -> None: pass
    def wear(self): pass

class AwesomeBoots(Awesome, Boots):
    pass

def main() -> None:
    myboots = AwesomeBoots(color='red')
    myboots.wear()
    myboots.sparkle(5)
    reveal_type(AwesomeBoots.__init__)

if __name__ == '__main__':
    main()

"""
<output>
multiple_inheritance.py:28: note: Revealed type is "def (self: __main__.Boots, color: builtins.str)"
</output>
"""



"""Interface implements attribute as property
"""
import zope.interface


class IBookmark(zope.interface.Interface):
    number: int
    text: str

@zope.interface.implementer(IBookmark)
class Bookmark(object):
    @property
    def number(self) -> str:
        return "hello"

    @number.setter
    def number(self, value: str) -> None:
        pass

    @property
    def text(self) -> int:
        return 5


def main() -> None:
    bm: IBookmark = Bookmark()
    bm.number = 12

"""
Properties are not type-checked in current version, so no error output
<output>
</output>
"""



"""A simple valid interface declaration
"""

import zope.interface


class ISomething(zope.interface.Interface):
    def hello(x: int, y: str) -> None:
        pass


@zope.interface.implementer(ISomething)
class Something(object):
    def hello(self, x: int, y: str) -> None:
        print(f"X: {x}, Y: {y}")


def run(smth: ISomething):
    smth.hello(1, "test")


def main() -> None:
    smth = Something()
    run(smth)


if __name__ == '__main__':
    main()

"""
<output>
</output>
"""



"""Incorrectly implemented interfaces are reported"""

from zope.interface import implementer
from zope.interface import Interface
from zope.interface import classImplements

from unknown import Foo


class IFoo(Interface):
    def foo() -> int:
        pass


classImplements(Foo, IFoo)


"""
<output>
classimplements_broken_impl.py:7: error: Cannot find implementation or library stub for module named "unknown"
classimplements_broken_impl.py:7: note: See https://mypy.readthedocs.io/en/stable/running_mypy.html#missing-imports
classimplements_broken_impl.py:15: error: __main__.Foo is not a class, cannot mark it as a interface implementation
</output>
"""



from zope.interface import implementer
from zope.interface import Interface
from zope.interface import classImplements


class IFoo(Interface):
    def foo() -> int:
        pass


class IBar(Interface):
    def bar() -> str:
        pass


class FooBar:
    def foo(self) -> int:
        return 0

    def bar(self) -> str:
        return ""


classImplements(FooBar, IFoo, IBar)


foo: IFoo = FooBar()
bar: IBar = FooBar()


"""
<output>
</output>
"""



"""zope.interface provides an annotated attribute for classes
"""
import typing
import zope.interface

class IFoo(zope.interface.Interface):
    f_str: typing.Text = zope.interface.Attribute("Text Attr")
    f_str_opt: typing.Optional[typing.Text] = zope.interface.Attribute("Optional Text Attr")
    f_int: int = zope.interface.Attribute("Int Attr")
    f_bar: "IBar" = zope.interface.Attribute("Other Intarface Attr")

class IBar(zope.interface.Interface):
    pass

@zope.interface.implementer(IFoo)
class Foo(object):
    f_str = None
    f_str_opt = None
    f_int = None
    f_bar = None

@zope.interface.implementer(IBar)
class Bar(object):
    pass

def main() -> None:
    foo: IFoo = Foo()
    bar: IBar = Bar()

    foo.f_str = "Sample"
    foo.f_str = 10
    foo.f_str = None
    foo.f_str_opt = None
    foo.f_int = 10
    foo.f_bar = bar

if __name__ == '__main__':
    main()

"""
<output>
interface_annotated_attribute.py:31: error: Incompatible types in assignment (expression has type "int", variable has type "str")
interface_annotated_attribute.py:32: error: Incompatible types in assignment (expression has type "None", variable has type "str")
</output>
"""



# Stubs for zope.schema._bootstrapinterfaces (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import zope.interface
from typing import Any, Optional

class StopValidation(Exception): ...

class ValidationError(zope.interface.Invalid):
    field: Any = ...
    value: Any = ...
    def with_field_and_value(self, field: Any, value: Any): ...
    def doc(self): ...
    def __lt__(self, other: Any): ...
    def __eq__(self, other: Any): ...
    __hash__: Any = ...

class RequiredMissing(ValidationError):
    __doc__: Any = ...

class WrongType(ValidationError):
    __doc__: Any = ...
    expected_type: Any = ...
    value: Any = ...
    def __init__(self, value: Optional[Any] = ..., expected_type: Optional[Any] = ..., name: Optional[Any] = ..., *args: Any) -> None: ...

class OutOfBounds(ValidationError):
    bound: Any = ...
    TOO_LARGE: Any = ...
    TOO_SMALL: Any = ...
    violation_direction: Any = ...
    value: Any = ...
    def __init__(self, value: Optional[Any] = ..., bound: Optional[Any] = ..., *args: Any) -> None: ...

class OrderableOutOfBounds(OutOfBounds): ...

class TooBig(OrderableOutOfBounds):
    __doc__: Any = ...
    violation_direction: Any = ...

class TooSmall(OrderableOutOfBounds):
    __doc__: Any = ...
    violation_direction: Any = ...

class LenOutOfBounds(OutOfBounds): ...

class TooLong(LenOutOfBounds):
    __doc__: Any = ...
    violation_direction: Any = ...

class TooShort(LenOutOfBounds):
    __doc__: Any = ...
    violation_direction: Any = ...

class InvalidValue(ValidationError):
    __doc__: Any = ...

class ConstraintNotSatisfied(ValidationError):
    __doc__: Any = ...

class NotAContainer(ValidationError):
    __doc__: Any = ...

class NotAnIterator(ValidationError):
    __doc__: Any = ...

class WrongContainedType(ValidationError):
    __doc__: Any = ...
    errors: Any = ...
    def __init__(self, errors: Optional[Any] = ..., name: Optional[Any] = ..., *args: Any) -> None: ...

class SchemaNotCorrectlyImplemented(WrongContainedType):
    __doc__: Any = ...
    schema_errors: Any = ...
    invariant_errors: Any = ...
    def __init__(self, errors: Optional[Any] = ..., name: Optional[Any] = ..., schema_errors: Optional[Any] = ..., invariant_errors: Any = ..., *args: Any) -> None: ...

class SchemaNotFullyImplemented(ValidationError):
    __doc__: Any = ...

class SchemaNotProvided(ValidationError):
    __doc__: Any = ...
    schema: Any = ...
    value: Any = ...
    def __init__(self, schema: Optional[Any] = ..., value: Optional[Any] = ..., *args: Any) -> None: ...

class NotAnInterface(WrongType, SchemaNotProvided):
    expected_type: Any = ...
    def __init__(self, value: Any, name: Any) -> None: ...

class IFromUnicode(zope.interface.Interface):
    def fromUnicode(value: Any) -> None: ...

class IFromBytes(zope.interface.Interface):
    def fromBytes(value: Any) -> None: ...

class IContextAwareDefaultFactory(zope.interface.Interface):
    def __call__(context: Any) -> None: ...

class IBeforeObjectAssignedEvent(zope.interface.Interface):
    object: Any = ...
    name: Any = ...
    context: Any = ...

class IValidatable(zope.interface.Interface):
    def validate(value: Any) -> None: ...

class NO_VALUE: ...



# Stubs for zope.interface.registry (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Generic, Iterator, Optional, Type, TypeVar, Tuple

T = TypeVar("T")

class _UnhashableComponentCounter:
    def __init__(self, otherdict: Any) -> None: ...
    def __getitem__(self, key: Any): ...
    def __setitem__(self, component: Any, count: Any): ...
    def __delitem__(self, component: Any): ...

class _UtilityRegistrations:
    def __init__(self, utilities: Any, utility_registrations: Any) -> None: ...
    def registerUtility(self, provided: Any, name: Any, component: Any, info: Any, factory: Any) -> None: ...
    def unregisterUtility(self, provided: Any, name: Any, component: Any) -> None: ...

class Components:
    __name__: Any = ...
    __bases__: Any = ...
    def __init__(self, name: str = ..., bases: Any = ...) -> None: ...
    def __reduce__(self): ...
    def registerUtility(self, component: Optional[Any] = ..., provided: Optional[Any] = ..., name: str = ..., info: str = ..., event: bool = ..., factory: Optional[Any] = ...): ...
    def unregisterUtility(self, component: Optional[Any] = ..., provided: Optional[Any] = ..., name: str = ..., factory: Optional[Any] = ...): ...
    def registeredUtilities(self) -> None: ...
    def queryUtility(self, provided: Type[T], name: str = ..., default: Optional[Any] = ...) -> Optional[T]: ...
    def getUtility(self, provided: Type[T], name: str = ...) -> T: ...
    def getUtilitiesFor(self, interface: Type[T]) -> Iterator[Tuple[str, T]]: ...
    def getAllUtilitiesRegisteredFor(self, interface: Any): ...
    def registerAdapter(self, factory: Any, required: Optional[Any] = ..., provided: Optional[Any] = ..., name: str = ..., info: str = ..., event: bool = ...) -> None: ...
    def unregisterAdapter(self, factory: Optional[Any] = ..., required: Optional[Any] = ..., provided: Optional[Any] = ..., name: str = ...): ...
    def registeredAdapters(self) -> None: ...
    def queryAdapter(self, object: Any, interface: Type[T], name: str = ..., default: Optional[Any] = ...) -> Optional[T]: ...
    def getAdapter(self, object: Any, interface: Type[T], name: str = ...) -> T: ...
    def queryMultiAdapter(self, objects: Any, interface: Type[T], name: str = ..., default: Optional[Any] = ...) -> Optional[T]: ...
    def getMultiAdapter(self, objects: Any, interface: Type[T], name: str = ...) -> T: ...
    def getAdapters(self, objects: Any, provided: Type[T]) -> Iterator[Tuple[str, T]]: ...
    def registerSubscriptionAdapter(self, factory: Any, required: Optional[Any] = ..., provided: Optional[Any] = ..., name: str = ..., info: str = ..., event: bool = ...) -> None: ...
    def registeredSubscriptionAdapters(self) -> None: ...
    def unregisterSubscriptionAdapter(self, factory: Optional[Any] = ..., required: Optional[Any] = ..., provided: Optional[Any] = ..., name: str = ...): ...
    def subscribers(self, objects: Any, provided: Type[T]) -> T: ...
    def registerHandler(self, factory: Any, required: Optional[Any] = ..., name: str = ..., info: str = ..., event: bool = ...) -> None: ...
    def registeredHandlers(self) -> None: ...
    def unregisterHandler(self, factory: Optional[Any] = ..., required: Optional[Any] = ..., name: str = ...): ...
    def handle(self, *objects: Any) -> None: ...

class UtilityRegistration:
    def __init__(self, registry: Any, provided: Any, name: Any, component: Any, doc: Any, factory: Optional[Any] = ...) -> None: ...
    def __hash__(self): ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    def __lt__(self, other: Any): ...
    def __le__(self, other: Any): ...
    def __gt__(self, other: Any): ...
    def __ge__(self, other: Any): ...

class AdapterRegistration:
    def __init__(self, registry: Any, required: Any, provided: Any, name: Any, component: Any, doc: Any) -> None: ...
    def __hash__(self): ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    def __lt__(self, other: Any): ...
    def __le__(self, other: Any): ...
    def __gt__(self, other: Any): ...
    def __ge__(self, other: Any): ...

class SubscriptionRegistration(AdapterRegistration): ...

class HandlerRegistration(AdapterRegistration):
    def __init__(self, registry: Any, required: Any, name: Any, handler: Any, doc: Any) -> None: ...
    @property
    def factory(self): ...
    provided: Any = ...



"""Method parameters refers to yet undeclared type
"""

import zope.interface

class ISomething(zope.interface.Interface):
    def hello(thing: "IThing") -> None:
        pass

@zope.interface.implementer(ISomething)
class Something(object):
    def hello(self, thing: "IThing") -> None:  # Wrong kind of list
        print(f"Hello, {thing.get_name()}")
        reveal_type(thing)


class IThing(zope.interface.Interface):
    def get_name() -> str:
        pass

"""
<output>
forwardref.py:14: note: Revealed type is "__main__.IThing"
</output>
"""



# Stubs for zope.interface.advice (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def getFrameInfo(frame: Any): ...
def addClassAdvisor(callback: Any, depth: int = ...): ...
def isClassAdvisor(ob: Any): ...
def determineMetaclass(bases: Any, explicit_mc: Optional[Any] = ...): ...
def minimalBases(classes: Any): ...



"""
Reproduces a bug where MROs were incorrectly computed for implementers
    https://github.com/Shoobx/mypy-zope/issues/34
    https://github.com/Shoobx/mypy-zope/issues/76
"""

from zope.interface import implementer, Interface


class IProtocol(Interface):
    pass


def main() -> None:
    class Factory:
        # It seems important for "Protocol" to show up as an attribute annotation to
        # trigger the bug(!?)
        protocol: "Protocol"

    @implementer(IProtocol)
    class Protocol:
        pass


if __name__ == '__main__':
    main()

"""
Expect no errors. A specific test checks that we correct compute the MRO of `Protocol`.
<output>
</output>
"""



"""When importing an interface from unknown module, we just ignore it
"""
import zope.interface

from unknown import IUnknownInterface

@zope.interface.implementer(IUnknownInterface)
class Bookmark(object):
    flag = True

def main() -> None:
    bm = Bookmark()
    bm.flag = False

if __name__ == '__main__':
    main()

"""
<output>
unknown_interface.py:5: error: Cannot find implementation or library stub for module named "unknown"
unknown_interface.py:5: note: See https://mypy.readthedocs.io/en/stable/running_mypy.html#missing-imports
</output>
"""
